//********************************************************************************
// Projeto - controle remoto com módulo RF 315MHz + HT12E
//          
// Simula o MSX no controle do robô ADR1-R usando botões!
//
// Desenvolvido por: Eng. Márcio José Soares
// Versão: 1.0
//
// Plataforma: Arduino Nano
// Módulo transmissor - TXC1 315MHz
// Frequência: 315MHz
//
// Pinos utilizado:
//
// 13 - LEDLive
//
// 2 - pinD0 (btn up)
// 3 - pinD1 (btn down)
// 4 - pinD2 (btn right)
// 5 - pinD3 (btn left)
//
// 9  - pinA4
// 10 - pinA5
// 11 - pinA6
// 12 - pinA7
//
// A0 - btnUP
// A1 - btnDOWN
// A2 - btnRIGHT
// A3 - btnLEFT
//
//********************************************************************************

//********************************************************************************
// Arquivos incluídos no módulo
//********************************************************************************
#include "pindeclarations.h"

//********************************************************************************
// Definições para pinos de I/O
//********************************************************************************
#define LEDLive   PB5
#define pinD0     PD2
#define pinD1     PD3
#define pinD2     PD4
#define pinD3     PD5

#define pinA4     PB1
#define pinA5     PB2
#define pinA6     PB3
#define pinA7     PB4

#define btnUP     PC0
#define btnDOWN   PC1   
#define btnRIGHT  PC2
#define btnLEFT   PC3
//
//********************************************************************************
// Definições importantes do módulo
//********************************************************************************
#define TRUE    1
#define FALSE   0

#define CFWD       1
#define CRWD       2
#define CRIGHT     4
#define CLEFT      8
#define CSFRIGHT   5 
#define CSFLEFT    9
#define CSRRIGHT   6
#define CSRLEFT    10
#define CSTOP      0

#define ENDADR1R   0x02

//********************************************************************************
// Variáveis globais do módulo
//********************************************************************************
int LEDliveState = LOW;
long interval = 1;                        //ms  - tempo para intervalo
long previous = 0;
int my_pos = 0;

//********************************************************************************
// Constantes globais do módulo
//********************************************************************************
const int LEDTIME = 250;                  //ms  - 250 x 2ms  = 500ms
const int INSTIME = 125;                  //ms  - 125 x 2ms = 250ms
const int TIMEOUT = 1500;                 //ms  - 3000 x 2ms = 3000ms

//********************************************************************************
// Setup - Arduino
//********************************************************************************
void setup() {
    //configura porta para debug 
    Serial.begin(2400);
  
    //define direção dos pinos
    pinMode(LEDLive,OUTPUT);
    
    pinMode(pinD0,OUTPUT);
    pinMode(pinD1,OUTPUT);
    pinMode(pinD2,OUTPUT);
    pinMode(pinD3,OUTPUT);

    pinMode(pinA4,OUTPUT);
    pinMode(pinA5,OUTPUT);
    pinMode(pinA6,OUTPUT);
    pinMode(pinA7,OUTPUT);

    pinMode(btnUP,INPUT_PULLUP);
    pinMode(btnDOWN,INPUT_PULLUP);
    pinMode(btnRIGHT,INPUT_PULLUP);
    pinMode(btnLEFT,INPUT_PULLUP);

    digitalWrite(pinA4, LOW);         //configura para enviar para o robô! 
    digitalWrite(pinA5, HIGH);        //02H
    digitalWrite(pinA6, LOW);
    digitalWrite(pinA7, LOW);
}

//********************************************************************************
// Loop - Arduino
//********************************************************************************
void loop() {

    unsigned long current = 0;
    uint16_t timeLED = 0;
    uint16_t timeINs = 0;
    
    
    while(TRUE){
        
        current = millis();                       //tempo atual em ms
        if((current - previous) > interval)
        {
            previous = current;                   //salva tempo atual

            timeINs++;
            if(timeINs == INSTIME)
            {
                timeINs = 0;
                readButtons();
            }
            
            timeLED++;
            if(timeLED == LEDTIME)
            {
                timeLED = 0;  
            
                if(LEDliveState == LOW)           //troca estado atual do LED
                {
                    LEDliveState = HIGH;
                }
                else
                {
                    LEDliveState = LOW;
                }
            
                digitalWrite(LEDLive, LEDliveState);  
            }
       }
    }
}

//********************************************************************************
// Funcao para ler as chaves
//
// Entradas - nenhuma
// Saídas   - char com valor para enviar
//********************************************************************************
void readButtons(void){

    uint8_t myins = 0;
    int val = 0;
    
    val = digitalRead(btnUP);                 //lê pinos
    if(val == LOW)
       myins |= 0x01;
    else
       myins &= ~0x01;

    val = digitalRead(btnDOWN);                 //lê pinos
    if(val == LOW)
       myins |= 0x02;
    else
       myins &= ~0x02;


    val = digitalRead(btnRIGHT);                 //lê pinos
    if(val == LOW)
       myins |= 0x04;
    else
       myins &= ~0x04;


    val = digitalRead(btnLEFT);                 //lê pinos
    if(val == LOW)
       myins |= 0x08;
    else
       myins &= ~0x08;

    trataADR1R(myins);
}

//***********************************************************************
// Função trataADR1 - trata dados recebidos para ADR1
//
// Entradas - nenhuma
// Saiudas  - nenhuma
//***********************************************************************
void trataADR1R(uint8_t dados)
{
   
    switch(dados){
        case CFWD:
            digitalWrite(pinD0, LOW);
            digitalWrite(pinD1, HIGH);
            digitalWrite(pinD2, HIGH);
            digitalWrite(pinD3, HIGH);
            break;
        case CRWD:
            digitalWrite(pinD0, HIGH);
            digitalWrite(pinD1, LOW);
            digitalWrite(pinD2, HIGH);
            digitalWrite(pinD3, HIGH);
            break;
        case CRIGHT:
            digitalWrite(pinD0, HIGH);
            digitalWrite(pinD1, HIGH);
            digitalWrite(pinD2, LOW);
            digitalWrite(pinD3, HIGH);
            break;
        case CLEFT:
            digitalWrite(pinD0, HIGH);
            digitalWrite(pinD1, HIGH);
            digitalWrite(pinD2, HIGH);
            digitalWrite(pinD3, LOW);
            break;
        case CSFRIGHT:
            digitalWrite(pinD0, LOW);
            digitalWrite(pinD1, HIGH);
            digitalWrite(pinD2, LOW);
            digitalWrite(pinD3, HIGH);
            break; 
        case CSFLEFT:    
            digitalWrite(pinD0, LOW);
            digitalWrite(pinD1, HIGH);
            digitalWrite(pinD2, HIGH);
            digitalWrite(pinD3, LOW);
            break;
        case CSRRIGHT:
            digitalWrite(pinD0, HIGH);
            digitalWrite(pinD1, LOW);
            digitalWrite(pinD2, LOW);
            digitalWrite(pinD3, HIGH);
            break;  
        case CSRLEFT:    
            digitalWrite(pinD0, HIGH);
            digitalWrite(pinD1, LOW);
            digitalWrite(pinD2, HIGH);
            digitalWrite(pinD3, LOW);
            break;    
        case CSTOP:
            digitalWrite(pinD0, HIGH);
            digitalWrite(pinD1, HIGH);
            digitalWrite(pinD2, HIGH);
            digitalWrite(pinD3, HIGH);
            break;
        default:
            digitalWrite(pinD0, HIGH);
            digitalWrite(pinD1, HIGH);
            digitalWrite(pinD2, HIGH);
            digitalWrite(pinD3, HIGH);
    }
}
